GBankClassic_Chat = {}

function GBankClassic_Chat:Init()
    GBankClassic_Core:RegisterChatCommand("bank", function(input)
        return GBankClassic_Chat:ChatCommand(input)
    end)

    self.addon_outdated = false
    self.last_roster_sync = nil
    self.last_alt_sync = {}
    self.sync_queue = {}
    self.is_syncing = false

    GBankClassic_Core:RegisterComm("gbank-d", function (prefix, message, distribution, sender)
        GBankClassic_Chat:OnCommReceived(prefix, message, distribution, sender)
    end)

    GBankClassic_Core:RegisterComm("gbank-v", function (prefix, message, distribution, sender)
        GBankClassic_Chat:OnCommReceived(prefix, message, distribution, sender)
    end)

    GBankClassic_Core:RegisterComm("gbank-r", function (prefix, message, distribution, sender)
        GBankClassic_Chat:OnCommReceived(prefix, message, distribution, sender)
    end)
end

function GBankClassic_Chat:OnCommReceived(prefix, message, _, sender)
    local player = GBankClassic_Guild:GetPlayer()
    if player == sender then
        return
    end

    if prefix == "gbank-v" then
        local success, data = GBankClassic_Core:Deserialize(message)
        if success then
            local current_data = GBankClassic_Guild:GetVersion()
            if current_data then
                if data.name then
                    if current_data.name ~= data.name then
                        GBankClassic_Core:Print("A non-guild version!")
                        return
                    end
                end
                if data.addon then
                    if data.addon > current_data.addon then
                        if not self.addon_outdated then
                            -- only make the callout once
                            self.addon_outdated = true
                            GBankClassic_Core:Print("A newer version is available!")
                        end
                    end
                end
                if data.roster then
                    if current_data.roster == nil or data.roster > current_data.roster then
                        GBankClassic_Guild:RequestRosterSync(sender, data.roster)
                    end
                end
                if data.alts then
                    for k, v in pairs(data.alts) do
                        if not current_data.alts[k] or v > current_data.alts[k] then
                            GBankClassic_Guild:RequestAltSync(sender, k, v)
                        end
                    end
                end
            end
        end
    end

    if prefix == "gbank-r" then
        local success, data = GBankClassic_Core:Deserialize(message)
        if success then
            if data.player == player then
                if data.type == "roster" then
                    local time = GetServerTime()
                    if self.last_roster_sync == nil or time - self.last_roster_sync > 300 then
                        self.last_roster_sync = time
                        GBankClassic_Guild:SendRosterData()
                    end
                end

                if data.type == "alt" then
                    table.insert(self.sync_queue, data.name)
                    if not self.is_syncing then
                        GBankClassic_Chat:ProcessQueue()
                    end
                end
            end
        end
    end

    if prefix == "gbank-d" then
        local success, data = GBankClassic_Core:Deserialize(message)
        if success then
            if data.type == "roster" then
                GBankClassic_Guild:ReceiveRosterData(data.roster)
            end

            if data.type == "alt" then
                GBankClassic_Guild:ReceiveAltData(data.name, data.alt)
            end
        end
    end
end

function GBankClassic_Chat:ChatCommand(input)
    if input == nil or input == "" then
        GBankClassic_UI_Inventory:Toggle()
    else
        local commands = {
            ["sync"] = function ()
                GBankClassic_Events:Sync()
            end,
        }

        local prefix, _ = GBankClassic_Core:GetArgs(input, 1)
        local cmd = commands[prefix]
        if cmd ~= nil then
            cmd()
        else
            GBankClassic_UI_Inventory:Toggle()
        end
    end

    return false
end

function GBankClassic_Chat:ShowHelp()
    GBankClassic_Core:Print("No commands available")
end

function GBankClassic_Chat:ProcessQueue()
    if #self.sync_queue == 0 then
        self.is_syncing = false
        return
    end

    self.is_syncing = true

    local time = GetServerTime()

    local name = table.remove(self.sync_queue)
    if not self.last_alt_sync[name] or time - self.last_alt_sync[name] > 300 then
        self.last_alt_sync[name] = time
        GBankClassic_Guild:SendAltData(name)
    end

    GBankClassic_Chat:ReprocessQueue()
end

function GBankClassic_Chat:ReprocessQueue()
    GBankClassic_Core:ScheduleTimer(function (...) GBankClassic_Chat:OnTimer() end, 5)
end

function GBankClassic_Chat:OnTimer()
    GBankClassic_Chat:ProcessQueue()
end