GBankClassic_Guild = {}

GBankClassic_Guild.Info = nil

function GBankClassic_Guild:GetPlayer()
    return UnitName("player")
end

function GBankClassic_Guild:GetGuild()
    return IsInGuild("player") and GetGuildInfo("player") or nil
end

function GBankClassic_Guild:GetPlayerInfo(name)
    for i = 1, GetNumGuildMembers() do
        local playerRealm, _, _, _, _, _, _, _, _, _, class = GetGuildRosterInfo(i)
        player, _ = string.match(playerRealm, "(.*)%-(.*)")
        if player == name then
            return class
        end
    end
    return nil
end

function GBankClassic_Guild:Reset(name)
    if not name then return end

    GBankClassic_UI_Inventory:Close()
    GBankClassic_Database:Reset(name)
    self.Info = GBankClassic_Database:Load(name)
end

function GBankClassic_Guild:Init(name)
    if not name then return false end
    if self.Info and self.Info.name == name then return false end

    self.hasRequested = false
    self.requestCount = 0

    self.Info = GBankClassic_Database:Load(name)
    if self.Info then return true end

    self:Reset(name)
    return true
end

function GBankClassic_Guild:GetBanks()
    local hasBanks = false
    local banks = {}
    for i = 1, GetNumGuildMembers() do
        local name, _, _, _, _, _, _, officer_note, _, _, _ = GetGuildRosterInfo(i)
        if officer_note == "gbank" then
            local player, _ = string.match(name, "(.*)%-(.*)")
            table.insert(banks, player)
            hasBanks = true
        end
    end
    if not hasBanks then return nil end
    return banks
end

function GBankClassic_Guild:IsBank(player)
    local banks = GBankClassic_Guild:GetBanks()
    if banks == nil then return false end

    local isBank = false
    for _, v in pairs(banks) do
        if v == player then
            isBank = true
        end
    end

    return isBank
end

function GBankClassic_Guild:CheckVersion(version)
    if self.Info then return false end

    if version > self.Info.roster.version then
        return false
    end

    return true
end

function GBankClassic_Guild:GetVersion()
    if not self.Info then return nil end

    local data = {
        addon = tonumber(GetAddOnMetadata("GBankClassic", "Version")),
        roster = nil,
        alts = {}
    }

    if self.Info.name then
        data.name = self.Info.name
    end

    if self.Info.roster.version then
        data.roster = self.Info.roster.version
    end

    for k, v in pairs(self.Info.alts) do
        data.alts[k] = v.version
    end

    return data
end

function GBankClassic_Guild:RequestRosterSync(player, version)
    self.hasRequested = true
    self.requestCount = self.requestCount + 1
    local data = GBankClassic_Core:Serialize({player = player, type = "roster", version = version})
    GBankClassic_Core:SendCommMessage("gbank-r", data, "Guild", nil, "BULK")
end

function GBankClassic_Guild:RequestAltSync(player, name, version)
    self.hasRequested = true
    self.requestCount = self.requestCount + 1
    local data = GBankClassic_Core:Serialize({player = player, type = "alt", name = name, version = version})
    GBankClassic_Core:SendCommMessage("gbank-r", data, "Guild", nil, "BULK")
end

function GBankClassic_Guild:SendRosterData()
    local data = GBankClassic_Core:Serialize({type = "roster", roster = self.Info.roster})
    GBankClassic_Core:SendCommMessage("gbank-d", data, "Guild", nil, "BULK")
end

function GBankClassic_Guild:ReceiveRosterData(roster)
    if not self.Info then return end
    if self.Info.roster.version and roster.version < self.Info.roster.version then return end
    if self.hasRequested then
        self.requestCount = self.requestCount - 1
        if self.requestCount == 0 then
            self.hasRequested = false
            GBankClassic_Core:Print("Sync complete")
        end
    end

    self.Info.roster = roster
end

function GBankClassic_Guild:SendAltData(name)
    local data = GBankClassic_Core:Serialize({type = "alt", name = name, alt = self.Info.alts[name]})
    GBankClassic_Core:SendCommMessage("gbank-d", data, "Guild", nil, "BULK")
end

function GBankClassic_Guild:ReceiveAltData(name, alt)
    if not self.Info then return end
    if self.Info.alts[name] and alt.version < self.Info.alts[name].version then return end

    if self.hasRequested then
        self.requestCount = self.requestCount - 1
        if self.requestCount == 0 then
            self.hasRequested = false
            GBankClassic_Core:Print("Sync complete")
        end
    end

    self.Info.alts[name] = alt
end
